import os
import random
from collections import OrderedDict

from clinical_research_study_manager.add_patient_to_excel_file import add_patient
from clinical_research_study_manager.study_manager import get_base_project_directory


def get_int(x, y):
    return random.randint(x, y)


options = dict()
options['sex'] = ['M', 'F', 'U']
options['yes_no_unknown'] = ['Y', 'N', 'U']
options['yes_no_na'] = ['Y', 'N', 'NA']
options['reasons_not_eligible'] = ['unconsentable', 'does_not_meet_criteria', 'previously_enrolled', 'NA']
options['reasons_not_enrolled'] = ['declined', 'NA']
options['mrn'] = list(range(10000000, 20000000))
options['age'] = list(range(1, 100))
options['enrollment_arm'] = ['arm1', 'arm2']
options['name'] = ['jane', 'doe', 'bob', 'marley']


def get_options(options_dict=options):
    options_dict['date'] = ['{}/{}/{}'.format(get_int(1, 12),
                                              get_int(1, 28),
                                              get_int(2016, 2018)
                                              )
                            ]
    options_dict['time'] = ['{}:{}'.format(get_int(0, 23),
                                           get_int(0, 59)
                                           )
                            ]
    options_dict['initials'] = ['{}{}{}'.format(chr(get_int(97, 122)),
                                                chr(get_int(97, 122)),
                                                chr(get_int(97, 122))
                                                )
                                ]
    options['subject_id'] = [get_int(1, 1000)]
    return options


def get_screening_headers():
    screening_headers = OrderedDict()
    screening_headers['ScreeningDate'] = 'date'
    screening_headers['ScreeningTime'] = 'time'
    screening_headers['SubjectInitials'] = 'initials'
    screening_headers['MedicalRecordNumber'] = 'mrn'
    screening_headers['Age'] = 'age'
    screening_headers['Sex'] = 'sex'
    screening_headers['Eligible'] = 'yes_no_unknown'
    screening_headers['ReasonIneligible'] = 'reasons_not_eligible'
    screening_headers['Enrolled'] = 'yes_no_na'
    screening_headers['ReasonNotEnrolled'] = 'reasons_not_enrolled'
    screening_headers['ResearchAssistantInitials'] = 'initials'
    return screening_headers


def get_enrollment_headers():
    enrollment_headers = OrderedDict()
    enrollment_headers['SubjectID'] = 'subject_id'
    enrollment_headers['EnrollmentDate'] = 'date'
    enrollment_headers['EnrollmentTime'] = 'time'
    enrollment_headers['Age'] = 'age'
    enrollment_headers['Sex'] = 'sex'
    enrollment_headers['EnrollmentArm'] = 'enrollment_arm'
    enrollment_headers['ResearchAssistantInitials'] = 'initials'
    return enrollment_headers


def get_follow_up_headers():
    follow_up_headers = OrderedDict()
    follow_up_headers['SubjectID'] = 'subject_id'
    follow_up_headers['EnrollmentDate'] = 'date'
    follow_up_headers['EnrollmentTime'] = 'time'
    follow_up_headers['FollowUpDate'] = 'date'
    follow_up_headers['FollowUpTime'] = 'time'
    follow_up_headers['FollowUpComplete'] = 'yes_no_na'
    follow_up_headers['Notes'] = 'yes_no_na'
    return follow_up_headers


def get_linking_log_headers():
    linking_headers = OrderedDict()
    linking_headers['SubjectID'] = 'subject_id'
    linking_headers['MedicalRecordNumber'] = 'mrn'
    linking_headers['EnrollmentDate'] = 'date'
    linking_headers['EnrollmentTime'] = 'time'
    linking_headers['SubjectName'] = 'name'
    linking_headers['Age'] = 'age'
    linking_headers['Sex'] = 'sex'
    linking_headers['ResearchAssistantInitials'] = 'initials'
    return linking_headers


def generate_patient_dict(data_headers: OrderedDict) -> OrderedDict:
    options = get_options()
    patient_dict = OrderedDict()
    for header, data_type in data_headers.items():
        patient_dict[header] = random.choice(options[data_type])
    return patient_dict


def main():
    BASE_DIR = get_base_project_directory()
    BASE_DIR = os.path.join(BASE_DIR, 'Testing')
    screening_log_path = os.path.join(BASE_DIR, 'logs', 'Screening_Log.xlsx')
    enrollment_log_path = os.path.join(BASE_DIR, 'logs', 'Enrollment_Log.xlsx')
    follow_up_log = os.path.join(BASE_DIR, 'logs', 'Follow_Up_Log.xlsx')
    master_linking_log = os.path.join(BASE_DIR, 'logs', 'logs_with_phi', 'Master_Linking_Log.xlsx')

    data_files = [(screening_log_path, 'Screening_Log', get_screening_headers),
                  (enrollment_log_path, 'Enrollment_Log', get_enrollment_headers),
                  (follow_up_log, 'Follow_Up_Log', get_follow_up_headers),
                  (master_linking_log, 'Master_Linking_Log', get_linking_log_headers)
                  ]
    for data_file, data_sheet, data_func in data_files:
        print("writing {} file".format(data_file))
        for _ in range(100):
            patient_dict = generate_patient_dict(data_func())
            add_patient(data_file, patient_dict, data_sheet)


if __name__ == '__main__':
    main()



from . import add_patient_to_excel_file
from . import create_directories
from . import create_excel_files
from . import data_request_functions
from . import enrollment_queries
from . import enter_patient_on_log
from . import exceptions
from . import follow_up_queries
from . import generate_fake_data
from . import get_data_from_user
from . import load_project
from . import screening_queries
from . import stats_functions
from . import study_manager

# is this the correct way to be able to have it work by calling clinical_research_study_manager without having to call
# study manager also?
study_manager.start()



import argparse
import os
import sys

from clinical_research_study_manager import create_directories, create_excel_files, load_project

parser = argparse.ArgumentParser(prog='Research Study Manager',
                                 description='Command line interface to manage some common research tasks')
parser.add_argument('-create_project', metavar='Project_Name',
                    help='Creates a new project titled Project_Name in the Projects directory')
parser.add_argument('-load_project', metavar='Project_Name',
                    help='Loads Project Project_Name from the Projects directory for study activities')
parser.add_argument('-list_projects', action='store_true', help='List available projects to load')


def main():
    start()


def get_base_project_directory():
    """
    Creates the default project directories if needed and returns them
    :return: base directory where projects will be stored
    """
    if sys.platform in ('linux', 'darwin'):
        base_program_dir = os.path.join(os.environ['HOME'], 'Clinical_Research_Manager_Projects')
        base_project_dir = os.path.join(base_program_dir, 'Projects')
        if not os.path.exists(base_program_dir):
            os.mkdir(base_program_dir)
        if not os.path.exists(base_project_dir):
            os.mkdir(base_project_dir)
        return base_project_dir

    if sys.platform == 'win32':
        base_program_dir = os.path.join(os.environ['USERPROFILE'], 'Clinical_Research_Manager_Projects')
        base_project_dir = os.path.join(base_program_dir, 'Projects')
        if not os.path.exists(base_program_dir):
            os.mkdir(base_program_dir)
        if not os.path.exists(base_project_dir):
            os.mkdir(base_project_dir)
        return base_project_dir

    print("Can not determine system type")
    sys.exit(1)


def start():
    project_directory = get_base_project_directory()
    args = parser.parse_args()
    # Create new project
    if args.create_project is not None and args.create_project.strip():
        project_name = args.create_project.strip()
        project_path = os.path.join(project_directory, project_name)
        print("Creating new project titled {}".format(project_name))
        create_directories.create_project_directories(project_path, project_name)
        print("Created log files for project {}".format(project_name))
        create_excel_files.create_project_excel_files(project_path, project_name)
    # Bad entry
    elif args.create_project is not None and not args.create_project.strip():
        print("You must supply a non empty string")
        sys.exit(1)
    # load specific project
    if args.load_project is not None and args.load_project.strip():
        project_name = args.load_project.strip()
        project_path = os.path.join(project_directory, project_name)
        if os.path.exists(project_path) and os.path.isdir(project_path):
            load_project.load_project(project_path, project_name)
        else:
            print('Project does not exist')
            sys.exit(1)
    # Bad Entry
    elif args.load_project is not None and not args.load_project.strip():
        print("You must supply a non empty string")

    # Load list of projects for user to choose from
    if args.list_projects is True:
        print(os.listdir(project_directory))
        current_projects = {pid + 1: project
                            for pid, project in enumerate(os.listdir(project_directory))
                            if os.path.isdir(os.path.join(project_directory, project))
                            }
        # Create dict of projects to
        if current_projects:
            while True:
                print('Current Projects are: ')
                for pid, project_title in current_projects.items():
                    print('{} : {}'.format(pid, project_title))

                choice = (input('Please choose a project, q to quit. '))
                choice = int(choice) if choice.isnumeric() else choice
                if current_projects.get(choice):
                    project_title = current_projects[choice]
                    project_path = os.path.join(project_directory, project_title)
                    load_project.load_project(project_path, project_title)
                    sys.exit(0)
                elif choice.lower() == 'q':
                    sys.exit(0)
                else:
                    print('Please enter a valid choice')
                    continue


if __name__ == '__main__':
    main()



import os
from datetime import time

import matplotlib as plt
import pandas as pd

plt.rcParams['figure.figsize'] = (15, 15)


def print_stats(stat_type: str, patient_dict: dict):
    """
    Helper function for printing the stats from the patient data in a nice format
    :param stat_type: Stat type to print (Age, Race, Etc)
    :param patient_dict: dictionary containing stat data
    :return: No return
    """
    print("{} stats".format(stat_type), end="\n---------------------------\n")
    for stat, value in patient_dict.items():
        print(stat, value, sep=":")
    print()


def get_stats(log_path=None, log_sheet=None, log_type=None, data_frame=None):
    """
    Gets basic statistics for a log or a filtered DataFrame if given
    :param log_path: pathway to log to create stats from  (i.e. Screening, Enrollment)
    :param log_sheet: name of the sheet in the log that contains data (i.e. Screening_Loo, Enrollment_Log)
    :param log_type: type of log (i.e. Screening, Enrollment)
    :param data_frame: pre-filtered DataFrame generate stats for
    :return: No return
    """
    if log_path is not None and log_sheet is not None and log_type is not None:
        df = create_dataframe_from_log(log_path, log_sheet, log_type)
    else:
        df = data_frame
    # Total subjects
    if len(df) > 0:
        total_subjects_in_log = len(df)
        print("You have screened {} patients in total ".format(total_subjects_in_log))
        # Total by Sex
        get_stat_type(df, 'Sex')
        # Total by Age
        get_stat_type(df, 'Age')
        # Total Eligible
        get_stat_type(df, 'Eligible')
        # Reasons Ineligible
        get_stat_type(df, 'Reason_Ineligible')
        # Enrolled
        get_stat_type(df, 'Enrolled')
        # Reason Not Enrolled
        get_stat_type(df, 'Reason_Not_Enrolled')
    else:
        print("No subjects in log for that date")


def get_stat_type(df, stat_type):
    """
    Get statistics from a data frame for the given type if DataFrame contains that column
    :param df: DataFrame to get stats from
    :param stat_type: stat type to look for (i.e. Age, Gender)
    :return: No return
    """
    if stat_type == 'Age':
        if df.get('Age') is not None:
            age_stats = df['Age'].describe().to_dict()
            print_stats('Age', age_stats)
    elif df.get(stat_type) is not None:
        stat_value_counts = (df[stat_type].value_counts())
        print_stats(stat_type, stat_value_counts.to_dict())


def get_basic_stats_by_date(log_path, log_sheet, log_type, start_date, end_date):
    """
    Get basic statistics for a log filtered by date
    :param log_path: pathway to log to create stats from  (i.e. Screening, Enrollment)
    :param log_sheet: name of the sheet in the log that contains data (i.e. Screening_Loo, Enrollment_Log)
    :param log_type: type of log (i.e. Screening, Enrollment)
    :param start_date: initial date
    :param end_date: end date
    :return: data frame filtered on start_date and end_date
    """
    start_date = pd.to_datetime(start_date)
    end_date = pd.to_datetime(end_date)
    log_date = '{}Date'.format(log_type)
    df = create_dataframe_from_log(log_path, log_sheet, log_type)
    df[log_date] = df[log_date].apply(pd.to_datetime)
    df = df.loc[(df[log_date] >= start_date) & (df[log_date] <= end_date)]
    get_stats(data_frame=df)
    return df


def get_stats_by_time(log_path, log_sheet, log_type):
    """
    Get basic stats from a log by filtered by morning(7am-12pm), afternoon(12pm-4pm), evening(4pm-11pm),
    overnight(11pm-7am).
    :param log_path: pathway to log
    :param log_sheet: sheet in log where date is stored (i.e. Screening_Log)
    :param log_type: type of log your are searching (i.e. screening or enrollment)
    :return:
    """
    df = create_dataframe_from_log(log_path, log_sheet, log_type)
    morning = pd.to_datetime(str(time(7, 00).strftime('%H:%M')), format='%H:%M')
    afternoon = pd.to_datetime(str(time(12, 00).strftime('%H:%M')), format='%H:%M')
    evening = pd.to_datetime(str(time(16, 00).strftime('%H:%M')), format='%H:%M')
    night = pd.to_datetime(str(time(23, 00).strftime('%H:%M')), format='%H:%M')
    log_time = '{}Time'.format(log_type)
    df[log_time] = df[log_time].apply(pd.to_datetime, format='%H:%M')
    df_morning = df.loc[(df[log_time] >= morning) & (df[log_time] < afternoon)]
    df_afternoon = df.loc[(df[log_time] >= afternoon) & (df[log_time] < evening)]
    df_evening = df.loc[(df[log_time] >= evening) & (df[log_time] < night)]
    df_night = df.loc[(df[log_time] >= night) | (df[log_time] < morning)]
    # Morning Stats
    print("Stats for 07:00 to 12:00[Morning]")
    get_stats(data_frame=df_morning)
    print("-----------------------------", end="\n")
    print("Stats for 12:00 to 16:00[Afternoon]")
    get_stats(data_frame=df_afternoon)
    print("-----------------------------", end="\n")
    print("Stats for 16:00 to 23:00[Evening]")
    get_stats(data_frame=df_evening)
    print("-----------------------------", end="\n")
    print("Stats for 23:00 - 07:00[Overnight")
    get_stats(data_frame=df_night)
    return df_morning, df_afternoon, df_evening, df_night


def create_dataframe_from_log(log_path, log_sheet, log_type):
    """
    Create  a DataFrame from a given log
    :param log_path: pathway to log to create stats from  (i.e. Screening, Enrollment)
    :param log_sheet: name of the sheet in the log that contains data (i.e. Screening_Loo, Enrollment_Log)
    :param log_type: type of log (i.e. Screening, Enrollment)
    :return:
    """
    df = pd.read_excel(log_path)
    return df


def get_basic_plot(df, log_pathway, log_type):
    if len(df) > 0:
        # Get the date column we will use for various counts
        column_for_grouping = '{}Date'.format(log_type)
        # Add a date index to df
        df.set_index(df[column_for_grouping].apply(pd.to_datetime), inplace=True, drop=False)
        # Add Month, week and weekday columns
        df['Month'] = df.index.month
        df['Week'] = df.index.week  # Should we use week of year here?
        df['WeekDay'] = df.index.weekday_name
        # Create groups for plotting
        month = df.groupby('Month').size()
        # month.index = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
        week = df.groupby('Week').size()
        weekday = df.groupby('WeekDay').size()
        # Plot groups
        # Month
        data_viz_pathway = os.path.dirname(log_pathway).replace('logs', 'data_visualization')
        month_plot = month.plot(kind='bar')
        month_fig = month_plot.get_figure()
        month_figure_pathway = os.path.join(data_viz_pathway, '{}output_month.png'.format(log_type))
        month_fig.savefig(month_figure_pathway)
        print('Basic {} log by month chart saved to {}'.format(log_type, month_figure_pathway))

        # Week
        week_plot = week.plot(kind='bar')
        week_fig = week_plot.get_figure()
        week_figure_pathway = os.path.join(data_viz_pathway, '{}output_week.png'.format(log_type))
        week_fig.savefig(week_figure_pathway)
        print('Basic {} log by month chart saved to {}'.format(log_type, week_figure_pathway))

        # Weekday
        weekday_plot = weekday.plot(kind='bar')
        weekday_fig = weekday_plot.get_figure()
        weekday_figure_pathway = os.path.join(data_viz_pathway, '{}output_weekday.png'.format(log_type))
        weekday_fig.savefig(weekday_figure_pathway)
        print('Basic {} log by month chart saved to {}'.format(log_type, weekday_figure_pathway))


def main():
    pass


if __name__ == '__main__':
    main()



from clinical_research_study_manager import stats_functions, data_request_functions


def get_enrollment_log_stats_by_date(enrollment_log_path: str):
    """
    Get basic enrollment stats for subjects enrolled between two dates
    :return: DataFrame filtered on the two dates
    """
    start_date = data_request_functions.get_date_info('Start')
    end_date = data_request_functions.get_date_info('End')
    df = stats_functions.get_basic_stats_by_date(
        enrollment_log_path, 'Enrollment_Log', 'Enrollment', start_date, end_date)
    stats_functions.get_basic_plot(df, enrollment_log_path, 'Enrollment')


def get_enrollment_log_basic_stats(enrollment_log_path: str):
    """Get basic stats on enrolled patients including number enrolled, number by sex, age,
    """
    stats_functions.get_stats(log_path=enrollment_log_path, log_sheet='Enrollment_Log', log_type='Enrollment')


def get_enrollment_log_stats_by_time(enrollment_log_path: str):
    """Get basic stats on enrolled patients by morning, afternoon, evening and night."""
    stats_functions.get_stats_by_time(enrollment_log_path, 'Enrollment_Log', 'Enrollment')


def choose_query(enrollment_log_path: str):
    while True:
        # Ask for what the user would like to do
        print("1. Basic Enrollment Log Stats")
        print("2. Get basic Enrollment stats between two dates")
        print("3. Get basic Enrollment stats by time of day")
        choice = input("What actions would you like to take, q to quit ")

        choices = {'1': get_enrollment_log_basic_stats,
                   '2': get_enrollment_log_stats_by_date,
                   '3': get_enrollment_log_stats_by_time,
                   }

        if choice is not None and choice.strip() and choices.get(choice) is not None:
            choices[choice](enrollment_log_path)

        elif choice is not None and choice.strip() and choice.lower() == 'q':
            break
        # Bad Entry
        else:
            print("Please enter a valid choice")


def main():
    pass


if __name__ == '__main__':
    main()



import datetime

from clinical_research_study_manager.exceptions import InputException


def get_date_info(time_point: str):
    """
    Get date information for enrolled subject from user(i.e. follow up time)
    :param time_point: time point requested for user(i.e. enrollment vs follow up)
    :return: Screening date as datetime object
    """
    while True:
        try:
            value_from_user = input("What is the {} date MM/DD/YYYY [Leave Blank for Today] ".format(time_point))
            # TODO: add validation for correct date input
            if value_from_user and value_from_user.strip():  # Need better validation
                month, day, year = value_from_user.split("/")
                value_from_user = datetime.date(int(year), int(month), int(day))
                break
            if not value_from_user:
                value_from_user = datetime.date.today()
                break
            if not value_from_user.strip():  # Bad entry
                raise InputException(
                    "Please enter a valid {} date in the format MM/DD/YYYY or leave blank for today".format(time_point),
                    time_point)
        except ValueError:
            # TODO: add better error messages to show why this failed
            print("Please enter a valid {} date in the format MM/DD/YYYY".format(time_point))
            continue
    return value_from_user


def get_time_info(time_point: str):
    """
    Get time information for enrolled subject from user(i.e. enrollment)
    :param time_point: time point requested from user(enrollment vs follow up)
    :return: time point as datetime object
    """
    while True:
        try:
            value_from_user = input("What is the {} time HH:MM [Leave Blank for Today] ".format(time_point))
            # TODO: add validation for correct time input
            if value_from_user and value_from_user.strip():  # Need better validation
                hour, minute = value_from_user.split(":")
                value_from_user = datetime.time(int(hour), int(minute))
                return value_from_user
            if not value_from_user:
                string_time = datetime.datetime.now().strftime("%H:%M")
                hour, minute = string_time.split(":")
                value_from_user = datetime.time(int(hour), int(minute))
                return value_from_user
            if not value_from_user.strip():  # Bad entry
                raise InputException(
                    'Please enter a valid {} time in the format HH:MM or leave blank for now'.format(time_point),
                    time_point)
        except InputException as e:
            print(e)
        except ValueError:
            print("Please enter a valid {} time in the format HH:MM or leave blank for now".format(time_point))
            continue


# TODO: Add a validation dictionary for each type of input


def get_info_from_user(requested_info: str):
    """
    Get requested info for enrolled patient from user(i.e. subject_initials)
    :param requested_info: name of variable requested as str
    :return: value given by user as str
    """
    validators = dict()
    greater_than_zero = lambda x: int(x) > 0 if x.isnumeric() else False
    male_female = lambda x: x.lower() in ('m', 'f') if x.isalpha() else False
    yes_no = lambda x: x.lower() in ('y', 'n') if x.isalpha() else False

    validators['age'] = greater_than_zero
    validators['sex'] = male_female
    validators['enrollment status'] = yes_no
    validators['eligibility status'] = yes_no
    validators['follow up complete'] = yes_no

    while True:
        try:
            value_from_user = input("What is the {} for subject ".format(requested_info))
            # TODO: add validation for correct type input
            value_from_user = value_from_user.strip()
            if value_from_user:
                # Check to see if we need to do special validation
                if validators.get(requested_info) is not None:
                    if validators[requested_info](value_from_user):
                        return value_from_user
                    else:
                        raise InputException('Please enter a valid {} for subject'.format(requested_info),
                                             requested_info)
                else:
                    return value_from_user
            if not value_from_user:  # Bad entry
                raise InputException("Please enter a valid {} for subject".format(requested_info), requested_info)
        except InputException as e:
            print(e)
            continue



# TODO: Add custom exceptions for each input type i.e. make sure age isn't a negative number
# TODO: Also figure out how to map an exception to the input type. Do that in the class definition?


class MyException(ValueError):
    def __init__(self, msg):
        super().__init__(msg)


class HeaderException(MyException):
    def __init__(self, msg):
        super().__init__(msg)


class InputException(MyException):
    """
    Exceptions for different user inputs. Will display a different error message based on the input type.
    If input type is not one of the given exceptions will default to the error message in the called functions.
    """

    def __init__(self, msg, input_type):
        messages = dict()
        messages['age'] = 'Please enter a number greater than 0'
        messages['enrollment status'] = 'Please enter Y, N, NA'
        messages['eligibility status'] = 'Please enter Y, N, NA'
        messages['follow up complete'] = 'Please enter Y, N, NA'
        messages['sex'] = 'Please enter M, F, O, U'
        if messages.get(input_type) is not None:
            msg = messages[input_type]
        else:
            msg = msg
        super().__init__(msg)



from clinical_research_study_manager import data_request_functions, stats_functions


def get_screening_log_stats_by_date(screening_log_path: str):
    """
    Get basic stats from patients screened between two given dates
    :param screening_log_path: pathway to log
    :return: df filtered on the requested dates
    """
    start_date = data_request_functions.get_date_info('Start')
    end_date = data_request_functions.get_date_info('End')
    df = stats_functions.get_basic_stats_by_date(
        screening_log_path, 'Screening_Log', 'Screening', start_date, end_date)
    stats_functions.get_basic_plot(df, screening_log_path, 'Screening')
    return df


def get_screening_log_basic_stats(screening_log_path: str):
    """
    Get basic stats from the screening log including total screened, total by sex, age ranges,
    total eligible and reasons not eligible, total enrolled and reasons not enrolled.
    :param screening_log_path: pathway to log
    :return: No return
    """
    stats_functions.get_stats(log_path=screening_log_path, log_sheet='Screening_Log', log_type='Screening')


def get_screening_log_stats_by_time(screening_log_path: str):
    """
    Get basic stats based on morning, afternoon, evening and night
    :param screening_log_path: pathway to log
    :return: No Return
    """
    stats_functions.get_stats_by_time(screening_log_path, 'Screening_Log', 'Screening')


def choose_query(screening_log_path: str):
    while True:
        # Ask for what the user would like to do
        print("1. Basic Screening Log Stats")
        print("2. Get basic stats between two dates")
        print("3. Get basic stats by time of day")
        choice = input("What actions would you like to take, q to quit ")

        choices = {'1': get_screening_log_basic_stats,
                   '2': get_screening_log_stats_by_date,
                   '3': get_screening_log_stats_by_time,
                   }

        if choice is not None and choice.strip() and choices.get(choice) is not None:
            choices[choice](screening_log_path)

        elif choice is not None and choice.strip() and choice.lower() == 'q':
            break
        # Bad Entry
        else:
            print("Please enter a valid choice")


def main():
    pass


if __name__ == '__main__':
    main()



import os
from collections import namedtuple

from clinical_research_study_manager import add_patient_to_excel_file, data_request_functions

# TODO: Create test to make sure tuple fields match header fields in all areas of program

ScreenedPatient = namedtuple("ScreenedPatient", ",".join(['ScreeningDate', 'ScreeningTime SubjectInitials',
                                                          'MedicalRecordNumber', 'Age', 'Sex', 'Eligible',
                                                          'ReasonIneligible', 'Enrolled',
                                                          'ReasonNotEnrolled', 'ResearchAssistantInitials'
                                                          ]))
EnrolledPatient = namedtuple("EnrolledPatient",
                             ",".join(['SubjectID', 'EnrollmentDate', 'EnrollmentTime', 'Age', 'Sex',
                                       'EnrollmentArm', 'ResearchAssistantInitials'
                                       ]))
LinkedPatient = namedtuple("LinkedPatient", ",".join(['SubjectID', 'MedicalRecordNumber', 'EnrollmentDate',
                                                      'EnrollmentTime', 'SubjectName', 'Age', 'Sex',
                                                      'ResearchAssistantInitials'
                                                      ]))
FollowUpPatient = namedtuple("FollowUpPatient", ",".join(['SubjectID', 'EnrollmentDate', 'EnrollmentTime',
                                                          'FollowUpDate', 'FollowUpTime', 'FollowUpComplete',
                                                          'Notes'
                                                          ]))


def to_dict(func: namedtuple):
    """Converts a named tuple to its order dictionary form"""

    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)._asdict()

    return wrapper


@to_dict
def get_screened_patient_data() -> ScreenedPatient:
    """
    Get screening data for a single screened subject
    :return: Named Tuple with subjects data
    """
    patient_data = dict()
    patient_data['ScreeningDate'] = data_request_functions.get_date_info('screening')
    patient_data['ScreeningTime'] = data_request_functions.get_time_info('screening')
    patient_data['SubjectInitials'] = data_request_functions.get_info_from_user('subject initials')
    patient_data['MedicalRecordNumber'] = data_request_functions.get_info_from_user('medical record number')
    patient_data['Age'] = data_request_functions.get_info_from_user('age')
    patient_data['Sex'] = data_request_functions.get_info_from_user('sex')
    patient_data['Eligible'] = data_request_functions.get_info_from_user('eligibility status')
    patient_data['ReasonIneligible'] = data_request_functions.get_info_from_user('reason ineligible')
    patient_data['Enrolled'] = data_request_functions.get_info_from_user('enrollment status')
    patient_data['ReasonNotEnrolled'] = data_request_functions.get_info_from_user('reason not enrolled')
    patient_data['ResearchAssistantInitials'] = data_request_functions.get_info_from_user(
        'research assistant initials')

    return ScreenedPatient(**patient_data)


@to_dict
def get_enrolled_patient_data() -> EnrolledPatient:
    """
    Get enrollment data for an enrolled subject
    :return: Named Tuple with subjects data
    """
    patient_data = dict()
    patient_data['SubjectID'] = data_request_functions.get_info_from_user('subject id')
    patient_data['EnrollmentDate'] = data_request_functions.get_date_info('enrollment')
    patient_data['EnrollmentTime'] = data_request_functions.get_time_info('enrollment')
    patient_data['Age'] = data_request_functions.get_info_from_user('age')
    patient_data['Sex'] = data_request_functions.get_info_from_user('sex')
    patient_data['EnrollmentArm'] = data_request_functions.get_info_from_user('enrollment arm')
    patient_data['ResearchAssistantInitials'] = data_request_functions.get_info_from_user(
        'research assistant initials')

    return EnrolledPatient(**patient_data)


@to_dict
def get_master_linking_log_data() -> LinkedPatient:
    """
    Get master linking data for an enrolled subject
    :return: Named Tuple with subjects data
    """
    patient_data = dict()
    patient_data['SubjectID'] = data_request_functions.get_info_from_user('subject_id')
    patient_data['MedicalRecordNumber'] = data_request_functions.get_info_from_user('medical_record_number')
    patient_data['EnrollmentDate'] = data_request_functions.get_date_info('enrollment')
    patient_data['EnrollmentTime'] = data_request_functions.get_time_info('enrollment')
    patient_data['SubjectName'] = data_request_functions.get_info_from_user('subject_name')
    patient_data['Age'] = data_request_functions.get_info_from_user('age')
    patient_data['Sex'] = data_request_functions.get_info_from_user('sex')
    patient_data['ResearchAssistantInitials'] = data_request_functions.get_info_from_user(
        'research assistant initials')

    return LinkedPatient(**patient_data)


@to_dict
def get_follow_up_data() -> FollowUpPatient:
    """
    Get follow up data for a single enrolled subject
    :return: Named Tuple with subjects data
    """
    patient_data = dict()
    patient_data['SubjectID'] = data_request_functions.get_info_from_user('subject id')
    patient_data['EnrollmentDate'] = data_request_functions.get_date_info('enrollment')
    patient_data['EnrollmentTime'] = data_request_functions.get_time_info('enrollment')
    patient_data['FollowUpDate'] = data_request_functions.get_date_info('follow up')
    patient_data['FollowUpTime'] = data_request_functions.get_time_info('follow up')
    patient_data['FollowUpComplete'] = data_request_functions.get_info_from_user('follow up complete')
    patient_data['Notes'] = data_request_functions.get_info_from_user('notes')
    return FollowUpPatient(**patient_data)


def main():
    screened_patient_data = get_screened_patient_data()
    add_patient_to_excel_file.add_patient(os.path.join('logs', 'Screening_Log.xlsx'), screened_patient_data,
                                          'Screening_Log')


if __name__ == '__main__':
    main()



import os

import clinical_research_study_manager as rsm


def enter_screened_patient(project_path: str, project_name: str):
    """
    Enter a patient on the screening log for the project
    :param project_path: pathway to project
    :param project_name: project name
    :return:
    """
    # Get Screening Log
    screening_log_path = os.path.join(project_path, 'logs', 'Screening_Log.xlsx')

    while True:
        choice = input("Would you like to enter a new patient onto the screening log [Y/N]? ")

        if choice and choice.strip() and choice.lower() == 'y':
            patient_data = rsm.get_data_from_user.get_screened_patient_data()
            rsm.add_patient_to_excel_file.add_patient(screening_log_path,
                                                      patient_data,
                                                      'Screening_Log')
            print("Successfully added patient to {} screening log".format(project_name))

        elif choice and choice.strip() and choice.lower() == 'n':
            break
        else:
            print("Please enter Y or N")


def enter_enrolled_patient(project_path: str, project_name: str):
    """
    Enter patient onto enrollment log for project
    :param project_path: pathway to project
    :param project_name: project name
    :return:
    """
    # Get Enrollment Log
    enrollment_log_path = os.path.join(project_path, 'logs', 'Enrollment_Log.xlsx')

    while True:
        choice = input("Would you like to enter a new patient onto the Enrollment log [Y/N]? ")

        if choice and choice.strip() and choice.lower() == 'y':
            patient_data = rsm.get_data_from_user.get_enrolled_patient_data()
            rsm.add_patient_to_excel_file.add_patient(enrollment_log_path,
                                                      patient_data,
                                                      'Enrollment_Log')
            print("Successfully added patient to {} Enrollment log".format(project_name))

        elif choice and choice.strip() and choice.lower() == 'n':
            break
        else:
            print("Please enter Y or N")


def enter_follow_up_patient(project_path: str, project_name: str):
    """
    Enter patient onto the follow up log for the project
    :param project_path: pathway to project
    :param project_name: project name
    :return:
    """
    # Get Follow Up Log
    follow_up_log_path = os.path.join(project_path, 'logs', 'Follow_Up_Log.xlsx')

    while True:
        choice = input("Would you like to enter a new patient onto the Follow Up log [Y/N]? ")

        if choice and choice.strip() and choice.lower() == 'y':
            patient_data = rsm.get_data_from_user.get_follow_up_data()
            rsm.add_patient_to_excel_file.add_patient(follow_up_log_path,
                                                      patient_data,
                                                      'Follow_Up_Log')
            print("Successfully added patient to {} Follow Up log".format(project_name))

        elif choice and choice.strip() and choice.lower() == 'n':
            break
        else:
            print("Please enter Y or N")


def enter_linking_log_patient(project_path: str, project_name: str):
    """
    Enter patient onto the linking log for project
    :param project_path: pathway to project
    :param project_name: project name
    :return:
    """
    # Get Linking Log
    linking_log_path = os.path.join(project_path, 'logs', 'logs_with_phi', 'Master_Linking_Log.xlsx')

    while True:
        choice = input("Would you like to enter a new patient onto the Linking log log [Y/N]? ")

        if choice and choice.strip() and choice.lower() == 'y':
            patient_data = rsm.get_data_from_user.get_master_linking_log_data()
            rsm.add_patient_to_excel_file.add_patient(linking_log_path,
                                                      patient_data,
                                                      'Master_Linking_Log')
            print("Successfully added patient to {} Master Linking log".format(project_name))

        elif choice and choice.strip() and choice.lower() == 'n':
            break
        else:
            print("Please enter Y or N")



import sys

import openpyxl
from clinical_research_study_manager.exceptions import HeaderException


def add_patient(excel_file: str, patient_data: dict, sheet_name: str):
    """
    Add patient data to the given file
    :param excel_file: path to excel file to append data to
    :param patient_data: list of patient data to append
    :param sheet_name: sheet name to append data to
    :return: No Return
    """
    data_work_book = openpyxl.load_workbook(excel_file)
    data_sheet = data_work_book[sheet_name]
    headers = [cell.value for cell in data_sheet[1]]
    patient_data_headers = list(patient_data.keys())
    try:
        if headers != patient_data_headers:
            raise HeaderException(
                'Your patient headers to not match your file header. Please check headers and try again')
        if headers == patient_data_headers:
            data_sheet.append([str(item) for item in list(patient_data.values())])
            data_work_book.save(excel_file)
    # TODO: Replace this with an exception
    except HeaderException as e:
        print(e)
        print(headers)
        print(list(patient_data_headers))
        sys.exit(1)
    except Exception as e:
        print("There was an {} error".format(e))
        sys.exit(1)



import os
import sys


def make_directory(project_path, directory_to_create, project_name):
    """
    Create a directory for the project in the projects directory
    :param project_path: pathway to project
    :param directory_to_create: name of directory to create
    :param project_name: project name
    :return:
    """
    directory_path = os.path.join(project_path, directory_to_create)
    try:
        if not os.path.exists(directory_path):
            os.mkdir(directory_path)
            print("Created {} for project {} at {}".format(directory_to_create, project_name, project_path))
        else:
            if os.path.isdir(directory_path):
                print("Directory {} already exist".format(directory_path))
    except PermissionError as e:
        print(e)
        sys.exit(1)


def create_project_directories(project_path, project_name):
    """
    Create patient data and log directories for a project
    :param project_path: pathway to project
    :param project_name: project name
    :return:
    """
    # Create New Project Directory
    if not os.path.exists(project_path):
        os.mkdir(project_path)
    else:
        print("Proejct Directory already exit at {}".format(project_path))
    directories = ['patient_data[phi]', 'patient_data[de_identified]', 'logs', 'logs/logs_with_phi',
                   'data_visualization']
    for directory_to_create in directories:
        make_directory(project_path, directory_to_create, project_name)


def main():
    pass


if __name__ == '__main__':
    main()



import os
import sys

import clinical_research_study_manager as rsm


def load_project(project_path: str, project_name: str):
    """
    Load a project to be managed for study tasks
    :param project_path: pathway to project
    :param project_name: name of the project
    :return:
    """
    if os.path.isdir(project_path):
        print("Opening {}".format(project_name))
        manage_project(project_path, project_name)
    else:
        print("{} does not exist")
        sys.exit(1)


def manage_project(project_path: str, project_name: str):
    """
    Perform basic tasks for a given project including entering patients on screening log, enrollment log
    follow up log, and master linking log.
    :param project_path: pathway to project
    :param project_name: naame of project
    :return:
    """
    choices = {'1': manage_logs,
               '2': manage_queries,
               }
    while True:
        print("1. Manage Logs")
        print("2. Query Logs")
        choice = input("Please choose what actions you would like to take, q to quit. ")

        if choice is not None and choice.strip() and choices.get(choice) is not None:
            choices[choice](project_path, project_name)
        elif choice is not None and choice.strip() and choice.lower() == 'q':
            break
        else:
            print("Please pick a valid option")


def manage_logs(project_path: str, project_name: str):
    while True:
        # Ask for what the user would like to do
        print("1. Enter Patients on Screening Log")
        print("2. Enter Patients on Enrollment Log")
        print("3. Enter Patients on Follow Up Log")
        print("4. Enter Patients on Master Linking Log")
        choice = input("What actions would you like to take, q to quit ")

        choices = {'1': rsm.enter_patient_on_log.enter_screened_patient,
                   '2': rsm.enter_patient_on_log.enter_enrolled_patient,
                   '3': rsm.enter_patient_on_log.enter_follow_up_patient,
                   '4': rsm.enter_patient_on_log.enter_linking_log_patient,
                   }

        if choice is not None and choice.strip() and choices.get(choice) is not None:
            choices[choice](project_path, project_name)

        elif choice is not None and choice.strip() and choice.lower() == 'q':
            break
        # Bad Entry
        else:
            print("Please enter a valid choice")


def manage_queries(project_path: str, project_name: str):
    project_log_path = os.path.join(project_path, 'logs')
    while True:
        # Ask for what the user would like to do
        print("1. Screening Queries")
        print("2. Enrollment Queries")
        print("3. Follow Up Queries")
        choice = input("What actions would you like to take, q to quit ")

        choices = {'1': (rsm.screening_queries.choose_query,
                         os.path.join(project_log_path, 'Screening_Log.xlsx')),
                   '2': (rsm.enrollment_queries.choose_query,
                         os.path.join(project_log_path, 'Enrollment_Log.xlsx')),
                   '3': (rsm.follow_up_queries.choose_query,
                         os.path.join(project_log_path, 'Follow_Up_Log.xlsx')),
                   }

        if choice is not None and choice.strip() and choices.get(choice) is not None:
            log_pathway = choices[choice][1]
            choices[choice][0](log_pathway)

        elif choice is not None and choice.strip() and choice.lower() == 'q':
            break
        # Bad Entry
        else:
            print("Please enter a valid choice")



import datetime

import pandas as pd

from clinical_research_study_manager.data_request_functions import get_date_info
from clinical_research_study_manager.stats_functions import create_dataframe_from_log


def follow_up_by_custom_dates(follow_up_log_path):
    """
    Find follow ups between a two dates
    :param follow_up_log_path: pathway to log
    :return: DataFrame filter on the two dates
    """
    start_date = get_date_info('Follow Up Start')
    end_date = get_date_info('Follow Up End')
    follow_up_df = follow_ups_scheduled_between_dates(follow_up_log_path, start_date=start_date, end_date=end_date)
    print("You have {} follow Ups scheduled between {} and {}".format(len(follow_up_df), start_date, end_date))
    print(follow_up_df.head())
    return follow_up_df


def follow_ups_scheduled_for_today(follow_up_log_path):
    """
    Get Follow Ups Scheduled for Today
    :param follow_up_log_path: pathway to log
    :return: DataFrame filtered on today
    """
    today = pd.to_datetime(datetime.date.today())
    type(today)
    follow_up_df = follow_up_scheduled_on_date(follow_up_log_path, follow_up_date=today)
    print("You have {} follow ups scheduled for today".format(len(follow_up_df)))
    print(follow_up_df)
    return follow_up_df


def follow_ups_scheduled_for_this_week(follow_up_log_path):
    """
    Get Follow Ups scheduled for this week
    :return:  DataFrame filtered on today
    """
    start_of_week = pd.to_datetime(datetime.date.today())
    end_of_week = pd.to_datetime(datetime.date.today() + datetime.timedelta(7))
    follow_up_df = follow_ups_scheduled_between_dates(follow_up_log_path,
                                                      start_date=start_of_week,
                                                      end_date=end_of_week)
    print("You have {} follow ups scheduled for this week".format(len(follow_up_df)))
    print(follow_up_df.head())
    return follow_up_df


def follow_up_scheduled_on_date(follow_up_log_path, follow_up_date=None):
    """
    Create follow up DataFrame filtered on a specific date
    :param follow_up_log_path: pathway to log
    :param follow_up_date: date to filter DataFrame on if given
    :return: Filtered DataFrame
    """
    if follow_up_date is None:
        follow_up_date = get_date_info('Follow Up')
        print(follow_up_date, type(follow_up_date))
    follow_up_df = create_dataframe_from_log(log_path=follow_up_log_path,
                                             log_sheet='Follow_Up_Log',
                                             log_type='FollowUp')
    today_df = follow_up_df.loc[(follow_up_df.FollowUpDate == pd.to_datetime(follow_up_date))]
    print("You have {} follow up scheduled on {}".format(len(today_df), follow_up_date))
    print(today_df.head())
    return today_df


def follow_ups_scheduled_between_dates(follow_up_log_path, start_date=None, end_date=None):
    """
    Create Follow up DataFrame filtered between two dates
    :param follow_up_log_path: pathway to log
    :param start_date: start date if given
    :param end_date: end date if given
    :return: Filtered DataFrame
    """
    if start_date is None and end_date is None:
        start_date = get_date_info('Start')
        end_date = get_date_info('End')
    follow_up_df = create_dataframe_from_log(log_path=follow_up_log_path,
                                             log_sheet='Follow_Up_Log',
                                             log_type='FollowUp')
    week_df = follow_up_df.loc[(follow_up_df.FollowUpDate >= pd.to_datetime(start_date))
                               & (follow_up_df.FollowUpDate <= pd.to_datetime(end_date))]
    print("You have {} follow ups scheduled between {} and {}".format(len(week_df), start_date, end_date))
    print(week_df.head())
    return week_df


def patients_at_risk_of_being_lost(follow_up_log_path):
    """
    Create Follow up data frame for at risk subjects5
    :return: DataFrame containing subjects at risks of being lost to follow up
    """
    today = pd.to_datetime(datetime.date.today())
    follow_up_df = create_dataframe_from_log(log_path=follow_up_log_path,
                                             log_sheet='Follow_Up_Log',
                                             log_type='FollowUp')
    at_risk_df = follow_up_df.loc[(follow_up_df.FollowUpDate < today) &
                                  (follow_up_df.FollowUpComplete == 'N')]
    print("There are {} subjects at risk of being lost".format(len(at_risk_df)))
    print(at_risk_df.head())
    return at_risk_df


def choose_query(follow_up_log_path: str):
    """
    Method to manage query options for a follow up log
    :param follow_up_log_path:
    :return: No Return
    """
    while True:
        # Ask for what the user would like to do
        print("1. Follow Ups Today")
        print("2. Follow Ups This Week")
        print("3. Follow Ups on a specific Date")
        print("4. Follow Ups Between two Dates")
        print("5. Patients at Risk of being lost to follow up")
        choice = input("What actions would you like to take, q to quit ")

        choices = {'1': follow_ups_scheduled_for_today,
                   '2': follow_ups_scheduled_for_this_week,
                   '3': follow_up_scheduled_on_date,
                   '4': follow_ups_scheduled_between_dates,
                   '5': patients_at_risk_of_being_lost,
                   }

        if choice is not None and choice.strip() and choices.get(choice) is not None:
            choices[choice](follow_up_log_path)

        elif choice is not None and choice.strip() and choice.lower() == 'q':
            break
        # Bad Entry
        else:
            print("Please enter a valid choice")


def main():
    pass


if __name__ == '__main__':
    main()



import os
from collections import OrderedDict

import openpyxl


def get_screening_headers() -> list:
    """
    Get headers for creating the project screening log
    :return: list containing the headers
    """
    screening_headers = OrderedDict()
    screening_headers['ScreeningDate'] = 'date'
    screening_headers['ScreeningTime'] = 'time'
    screening_headers['SubjectInitials'] = 'initials'
    screening_headers['MedicalRecordNumber'] = 'mrn'
    screening_headers['Age'] = 'age'
    screening_headers['Sex'] = 'sex'
    screening_headers['Eligible'] = 'yes_no_unknown'
    screening_headers['ReasonIneligible'] = 'reasons_not_eligible'
    screening_headers['Enrolled'] = 'yes_no_na'
    screening_headers['ReasonNotEnrolled'] = 'reasons_not_enrolled'
    screening_headers['ResearchAssistantInitials'] = 'initials'
    return list(screening_headers.keys())


def get_enrollment_headers() -> list:
    """
    Get enrollment headers for creating project enrollment log
    :return: list containing the enrollment headers
    """
    enrollment_headers = OrderedDict()
    enrollment_headers['SubjectID'] = 'subject_id'
    enrollment_headers['EnrollmentDate'] = 'date'
    enrollment_headers['EnrollmentTime'] = 'time'
    enrollment_headers['Age'] = 'age'
    enrollment_headers['Sex'] = 'sex'
    enrollment_headers['EnrollmentArm'] = 'enrollment_arm'
    enrollment_headers['ResearchAssistantInitials'] = 'initials'
    return list(enrollment_headers.keys())


def get_follow_up_headers() -> list:
    """
    Get follow up headers for creating project follow up log
    :return: list containing the follow up headers
    """
    follow_up_headers = OrderedDict()
    follow_up_headers['SubjectID'] = 'subject_id'
    follow_up_headers['EnrollmentDate'] = 'date'
    follow_up_headers['EnrollmentTime'] = 'time'
    follow_up_headers['FollowUpDate'] = 'date'
    follow_up_headers['FollowUpTime'] = 'time'
    follow_up_headers['FollowUpComplete'] = 'yes_no_na'
    follow_up_headers['Notes'] = 'yes_no_na'
    return list(follow_up_headers.keys())


def get_linking_log_headers() -> list:
    """
    Get linking log headers for creating project master linking log
    :return: list of headers for linking log
    """
    linking_headers = OrderedDict()
    linking_headers['SubjectID'] = 'subject_id'
    linking_headers['MedicalRecordNumber'] = 'mrn'
    linking_headers['EnrollmentDate'] = 'date'
    linking_headers['EnrollmentTime'] = 'time'
    linking_headers['SubjectName'] = 'name'
    linking_headers['Age'] = 'age'
    linking_headers['Sex'] = 'sex'
    linking_headers['ResearchAssistantInitials'] = 'initials'
    return list(linking_headers.keys())


def create_excel_file(project_path: str, project_name: str, file_name: str, headers: list):
    """
    Create an excel for the giving project
    :param project_name: project name
    :param project_path: pathway to project
    :param file_name: pathway to save excel file
    :param headers: headers for the excel file
    :return: No Return
    """
    if file_name != 'Master_Linking_Log.xlsx':
        excel_file_path = os.path.join(project_path, 'logs', file_name)
    else:
        excel_file_path = os.path.join(project_path, 'logs', 'logs_with_phi', file_name)

    if not os.path.exists(excel_file_path):
        new_work_book = openpyxl.Workbook()
        work_sheet = new_work_book.active
        work_sheet.title = file_name.replace('.xlsx', '')
        work_sheet.append(headers)
        new_work_book.save(os.path.join(excel_file_path))
        print("Created {} log for {} at {}".format(file_name, project_name, excel_file_path))
    else:
        print('file already exist at {}'.format(excel_file_path))


def create_project_excel_files(project_path: str, project_name: str):
    """
    Create Screening, Enrollment, Follow Up and Master Linking logs for the project
    :param project_name: project name
    :param project_path: pathway to project
    :return:
    """
    file_names_and_headers = {'Screening_Log.xlsx': get_screening_headers(),
                              'Enrollment_Log.xlsx': get_enrollment_headers(),
                              'Follow_Up_Log.xlsx': get_follow_up_headers(),
                              'Master_Linking_Log.xlsx': get_linking_log_headers(),
                              }
    for file_name, headers in file_names_and_headers.items():
        create_excel_file(project_path, project_name, file_name, headers)



import datetime
import unittest
from unittest import TestCase
from unittest.mock import patch

from clinical_research_study_manager import data_request_functions


class TestUserInput(TestCase):
    @patch('builtins.input', lambda x: '11/1/2017')
    def test_date_input(self):
        self.assertEqual(data_request_functions.get_date_info('Follow Up'), datetime.date(2017, 11, 1))

    @patch('builtins.input', lambda x: '23:30')
    def test_time_input(self):
        self.assertEqual(data_request_functions.get_time_info('Follow Up'), datetime.time(23, 30))

    @patch('builtins.input', lambda x: '25')
    def test_age_input(self):
        self.assertEqual(data_request_functions.get_info_from_user('age'), '25')

    @patch('builtins.input', lambda x: 'M')
    def test_sex_input(self):
        self.assertEqual(data_request_functions.get_info_from_user('sex'), 'M')

    @patch('builtins.input', lambda x: 'Y')
    def test_eligibility_status_input(self):
        self.assertEqual(data_request_functions.get_info_from_user('eligibility status'), 'Y')

    @patch('builtins.input', lambda x: 'NA')
    def test_reason_ineligible_input(self):
        self.assertEqual(data_request_functions.get_info_from_user('reason ineligible'), 'NA')

    @patch('builtins.input', lambda x: 'N')
    def test_enrollment_input(self):
        self.assertEqual(data_request_functions.get_info_from_user('enrollment status'), 'N')

    @patch('builtins.input', lambda x: 'Declined')
    def test_reason_not_enrolled_input(self):
        self.assertEqual(data_request_functions.get_info_from_user('reason not enrolled'), 'Declined')

    @patch('builtins.input', lambda x: 'ABC')
    def test_initials_input(self):
        self.assertEqual(data_request_functions.get_info_from_user('research assistant initials'), 'ABC')


# TODO: Figure out how to test bad input since we are using a while loop for that. Is that a bad idea?
# TODO: Figure out how to call test using setup.py

def main():
    unittest.main()


if __name__ == '__main__':
    main()
